from stoyled import *

def banner(version, color=True):
    """
    Banner, just for fanciness
    
        Parameters:
            version (int): Version of SéCh3r
        
        Returns:
            None: Literally None object :p
    """
    _banner = """
     /  __       ____
 ()  _ / () |)    __/ ,_
 /\ |/|     |/\     \/  |
/(_)|_/\___/|  |/\__/   |/ """[1:]
    if color:
        print(f"{rst}{bold}{_banner}{rst}  v{version} by --{bold}naryal2580{rst}")
    else:
        print(f"{_banner}  v{version} by --naryal2580")
    return None


def prnHeads(headers, color=True, isVuln=True):
    """
    Printing headers with fanciness
    
        Parameters:
            headers (dict/list): Headers based on condition
            color (bool): Shall color be printed while function is running
            isVuln (bool): Are passed headers vulnerability
        
        Returns:
            None: Literally None object :p
    """
    if isVuln:
        if type(headers) != list:
            for header in headers:
                print(bad(f'{header} -> {headers[header]}', color))
        else:
            for header in headers:
                print(bad(f'{header} -> Not Present', color))
    else:
        for header in headers:
            print(info(f'{header} -> {headers[header]}', color))
    return None



from webbrowser import open_new_tab as webrowser_openNewTab

def google(query):
    """
    Google for a query in a default web browser.
    
        Parameters:
            query (str): Query to be googled
        
        Returns:
            None: Literally None object :p
    """
    url = f'https://google.com#q={query}'
    webrowser_openNewTab(url)
    return None



#!/usr/bin/env python3

"""
     /  __       ____
 ()  _ / () |)    __/ ,_
 /\ |/|     |/\     \/  |
/(_)|_/\___/|  |/\__/   |/

Usage:
    sech3r [--verbose] [--searchForVuln] [--noRedirects] [--insecure] [--noColor]
    sech3r <urls>... [--verbose] [--searchForVuln] [--noRedirects] [--insecure] [--noColor]
    sech3r -h | --help
    sech3r -V | --version

Options:
    -h --help           Display help, basically this screen.
    -V --version        Display version number.
    <urls>               Optional URL(s) input from the Command-Line.
    -v --verbose        Show verbose output.
    -s --searchForVuln  Open Default WebBrowser, Googling for Vulnerabilities.
    -r --noRedirects    Do not follow HTTP-redirects.
    -i --insecure       Bypass TLS/SSL verification.
    -c --noColor        No Colours to be used for the Output.

Examples:
    sech3r demo.testfire.net
    sech3r demo.testfire.net -vs
    sech3r demo.testfire.net -vr
    sech3r demo.testfire.net -c
    sech3r -vsirc
"""


__author__ = "naryal2580"
__version__ = "4.6"


from secher import *


def main(urls=[], verbose=False, search4cves=False, noRedirects=False, insecure=False, color=True):
    """
    `main` Function of sech3r.

        Parameters:
            urls (list): List of URL strings
            verbose (bool): Is verbosity necessary
            search4cves (bool): Shall tool search for CVEs from disclosed version
            noRedirects (bool): Shall redirections occur, while requesting for headers
            insecure (bool): Ignore TLS/SSL warnings
            color (bool): Shall color be printed while function is running
        
        Returns:
            Nothing
    """
    if urls:
        print(takenInput(f"URL(s) separated with double <space> -> {'  '.join(urls)}", color))

    else:
        urls = coolInput('URL(s) separated with double <space>', color).split('  ')
    print(info(f'Started [at] {fetchFormatedTime()}  -> Now, Requesting', color), end='\n\n')

    for url in urls:
        if len(urls) > 1:
            print(info(f'Requesting -> {url}', color))
        url = validateUrl(url, color)
        if url.startswith('http://'):
            print(warn('Warning -> Crafting a non TLS request', color))
        if insecure:
            print(warn('Warning -> Bypassing TLS verification'))
        heads = getHeaders(url, noRedirects, insecure, color)
        if heads:
            if verbose:
                print(info('Response Headers -> below:', color))
                for head in heads:
                    print(takenInput(f'{head}: {heads[head]}', color))
            secHeads = checkSecHeads(heads)
            secHeadsPresent = secHeads[0]
            secHeadsNotPresent = secHeads[1]
            infoHeads = checkInfoHeads(heads)
            vulnHeads = infoHeads[0]
            infoHeads = infoHeads[1]
            if secHeadsPresent:
                prnHeads(secHeadsPresent, color, False)
            if secHeadsNotPresent:
                prnHeads(secHeadsNotPresent, color)
            if vulnHeads:
                if search4cves:
                    for vulnHead in vulnHeads:
                        google(f'{vulnHeads[vulnHead]} CVE')
                prnHeads(vulnHeads, color)
            if infoHeads:
                prnHeads(infoHeads, color, False)
            print()


def run():
    """
    `run` function of sech3r, function to be executed whenever sech3r is supposed to be executed. Does argument parsing, and controls exits too.
        
        Parameters:
            Nothing

        Returns:
            Nothing
    """
    from docopt import docopt
    args = docopt(__doc__, version='SéCh3r v{}'.format(__version__))
    color = True
    verbose = search4cves = noRedirects = insecure = False
    if args['--noColor']:
        color = False
    if args['--verbose']:
        verbose = True
    if args['--searchForVuln']:
        search4cves = True
    if args['--noRedirects']:
        noRedirects = True
    if args['--insecure']:
        insecure = True
    banner(__version__, color)
    if verbose:
        print(info('Verbosity -> Enabled', color))
        if color:
            print(info('Colorized Output -> Yeah'))
        else:
            print(info('Much fanciness -> Nope', False))
        if search4cves:
            print(info('Google for CVEs -> Yup!', color))
        else:
            print(info('Interested in CVEs -> Nah', color))
        if noRedirects:
            print(info('Follow Redirects -> No', color))
        else:
            print(info('Do Follow redirects -> Sure', color))
        if insecure:
            print(info('Bypass TLS/SSL verification -> Yea', color))
        else:
            print(info('Ignore TLS/SSL warnings -> Noo!', color))

    if args['<urls>']:
        main(args['<urls>'], verbose, search4cves, noRedirects, insecure, color)
    else:
        main([], verbose, search4cves, noRedirects, insecure, color)

    coolExit(0, color)


if __name__ == "__main__":
    run()


from .core import *
from .extra import *

__author__ = "naryal2580"
__version__ = "4.6"

del bad, blink, blue, blue_l, bold, core, cyan, cyan_l, dim, extra, good, green, green_l, hidden, invert, italic, name, NoRedirects, normal, purple, purple_l, red, red_l, request, rst, strike, style, uline, webrowser_openNewTab, white, white_l, yellow, yellow_l


from setuptools import setup, find_packages

with open("./README.md") as f:
    long_description = f.read()

setup(
    name='sech3r',
    version='4.6',
    author="Captain Nick Lucifer",
    author_email="naryal2580@gmail.com",
    url="https://github.com/naryal2580/sech3r",
    download_url='https://github.com/naryal2580/sech3r/tarball/master',
    description="HTTP Security Header Checker",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    py_modules=['sech3r'],
    install_requires=[
                        'stoyled',
                        'docopt'
                    ],
    classifiers=[
         "Programming Language :: Python :: 3",
         'Environment :: Console',
         "License :: OSI Approved :: MIT License",
         'Topic :: Security',
         "Operating System :: OS Independent",
     ],
    python_requires='>=3',
    entry_points={
        'console_scripts': [
            'sech3r = sech3r:run',
        ],
    }
 )



from urllib import request
from urllib.parse import urlparse
from ssl import _create_unverified_context
from .style import *


class NoRedirects(request.HTTPRedirectHandler):
    """
    Class to handle HTTP Rediect, to be installed via opener on request object.
    """
    def redirect_request(self, req, fp, code, msg, headers, newurl):
        return None



def validateUrl(url, color=True):
    """
    Returns url string after adding scheme if not present, and modofying scheme if not http.

        Parameters:
            url (str): URL string
            color (bool): Shall color be printed while function is running
        
        Returns:
            url (str): URL string after prepending or modifying scheme to http
    """
    parsedUrl = urlparse(url)
    if not parsedUrl.scheme:
        url = 'http://' + url
    elif parsedUrl.scheme not in ('http', 'https'):
        print(bad(f'Scheme `{parsedUrl.scheme}` does not looks like of `http`', color))
        print(info('Scheme modified to `http`', color))
        url = url.split('://')
        url[0] = 'http'
        url = '://'.join(url)
    return url


def getHeaders(url, noRedirects=False, insecure=False, color=True):
    """
    Returns HTTP Headers of a queried URL.

        Parameters:
            url (str): URL string
            noRedirects (bool): Shall redirection be followed
            insecure (bool): Ignore TLS/SSL warnings
            color (bool): Shall color be printed while function is running
        
        Returns:
            headers (dict): HTTP Headers of a queried URL
    """
    try:
        if noRedirects:
            opener = request.build_opener(NoRedirects)
            request.install_opener(opener)
        req = request.Request(
                                url,
                                data=None,
                                headers={'User-Agent': 'sech3r/4.2'}
                            )
        if not insecure:
            resp = request.urlopen(req)
        else:
            resp = request.urlopen(req, context=_create_unverified_context())
        if resp.url != url:
            if resp.url.startswith('https://'):
                print(good(f'Redirected to -> {resp.url}', color))
            else:
                print(info(f'Redirected to -> {resp.url}', color))
    except Exception as excptn:
        print(bad(str(excptn).replace(': ', ' -> '), color))
        if 'HTTP Error' in str(excptn):
            resp = excptn
        else:
            return {}
    return dict(resp.headers)


def checkSecHeads(headers):
    """
    Check for Security Headers.

        Parameters:
            headers (dict): HTTP Headers
        
        Returns:
            headersPresent (dict): HTTP security headers present with it's values
            headersNotPresent (list): HTTP security headers not present
    """
    headersPresent = {}
    headersNotPresent = []
    security_headers = [
                        'Referrer-Policy',
                        'X-XSS-Protection',
                        'Content-Security-Policy',
                        'X-Frame-Options',
                        'Strict-Transport-Security',
                        'X-Content-Type-Options',
                        'X-Permitted-Cross-Domain-Policies',
                        'Public-Key-Pins',
                        'Expect-CT',
                        'Feature-Policy',
                        'Report-To',
                        'NEL'
                       ]
    for security_header in security_headers:
        if security_header in headers:
            headersPresent[security_header] = headers[security_header]
        else:
            headersNotPresent.append(security_header)
    return headersPresent, headersNotPresent


def checkInfoHeads(headers):
    """
    Check for Informative Headers.

        Parameters:
            headers (dict): HTTP Headers
        
        Returns:
            disclosedOnes (dict): Informative HTTP headers with disclosed version
            undisclosedOnes (dict): Informative HTTP headers with version not disclosed
    """
    version_disclosure_headers = [
                                    'Server',
                                    'X-AspNet-Version',
                                    'X-Powered-By'
                                ]
    disclosedOnes = {}
    undisclosedOnes = {}
    for header in version_disclosure_headers:
        if header in headers.keys():
            if any(char.isdigit() for char in headers[header]):
                disclosedOnes[header] = headers[header]
            else:
                undisclosedOnes[header] = headers[header]
    return disclosedOnes, undisclosedOnes


